#ifndef BLMHISTORY_H
#define BLMHISTORY_H

#include <QMainWindow>
#include <QHash>
#include <tvariant.h>

class Label;
class DbThread;
class Image;
class Painter;

namespace Ui {
class MainWindow;
}

class BlmHistory : public QMainWindow
{
    Q_OBJECT
    
    friend class WindowsSaveRestore;

public:
    explicit BlmHistory(QWidget *parent = 0);
    ~BlmHistory();

    enum Type { None, Bergoz, Photodiode };
    
public slots:
    void getData();

    void setLiveMode(bool live);

    void setImageDynamicRange(bool dr);

    void setMinExpLog();

    void setEmptyValue();

signals:
    void progress(int step);

    void magnifierEnabled(bool);

private slots:
    void blmNamesReady(const TVariant& sl);

    void onProgressUpdate(int step);

    void sourceIsArchived(const QString &, int i, int j);

    void dbDataAvailable();

    void changePalette(bool);

    void updateLiveData();

    void updateCountdownTimer();

    void configTabChanged(int tabno);

    void exportData() const;

    void updateCustomRange();

    void enableCustomRange(bool enab);

    void setMaxSpinBoxValue(int);

    void setMaxValSliderValue(double);

    void resetMaxAndMinDoubleSpinboxes();

protected:
    void closeEvent(QCloseEvent *);

private:
    Ui::MainWindow *ui;

    Label *mImageLabel;

    Type mType;

    DbThread *mDbThread;

    int mTotSteps, mCurrentStep;

    QHash<QString, QStringList> mBlmNamesHash;

    /* used to store blm names in the right order */
    QVector<QStringList> mBlmNames;
};

#endif // BLMHISTORY_H
