#ifndef DBTHREAD_H
#define DBTHREAD_H

#include <QThread>
#include <QDateTime>
#include <QStringList>
#include <QHash>
#include <QReadWriteLock>
#include "hdbdata.h"

class DbDataCache;

class DbThread : public QThread
{
    Q_OBJECT
public:
    explicit DbThread(QObject *parent,
                      const QString& dbUser,
                      const QString& dbPass,
                      const QString& dbHost,
                      const QString &schema,
                      const int dbPort,
                      bool hdbpp);
    
    QString lastError() const { return m_lastErrorMessage; }

    bool dbError() const { return m_dbError; }

    QHash<QString, QList<HdbData> > attDataHash();

    int maxDataLen();

    double maximumValue() const { return mMaximumValue; }

    double minimumValue() const { return mMinimumValue; }

    double minimumPositive() const { return mMinimumPositive; }

    QDateTime startDateTime() const;

    QDateTime stopDateTime() const;

    /** \brief returns the list of the devices that are not present into the
     * hdb database
     *
     * @return the device name list of the tango devices that are not logged into
     *         the hdb, in the form domain/family/member
     */
    QStringList missingDevices() const { return mMissingDevices; }

protected:
    virtual void run();

signals:
    void databaseError(const QString& origin, const QString& error);

    void progress(int step);
    
public slots:
    void getData(const QStringList &devices, const
                 QString&attName,
                 const QDateTime& startDt,
                 const QDateTime& stopDt);

    void setAbortFlag();

private:
    QStringList mDeviceList, mMissingDevices;

    QString mAttName;
    QString m_lastErrorMessage;
    QDateTime mStartDt, mStopDt;

    DbDataCache *mDbDataCache;

    QString m_dbHost, m_dbPassword, m_dbUser, m_dbName, m_dbSchema;

    int m_dbPort;

    bool m_dbError, mAbort, m_hdbPP;

    double mMaximumValue, mMinimumValue, mMinimumPositive;

    QReadWriteLock mRWLock, mAbortRWLock;
};

#endif // DBTHREAD_H
