#include "blmhistory_constants.h"
#include "hdbdataaligner.h"
#include <math.h>
#include <QDateTime>
#include <QtDebug>

HdbDataAligner::HdbDataAligner()
{
    mMaximum = -100;
    mMinimum = mMinimumPositive = 1000;
}

void HdbDataAligner::alignToTimestamps(QHash<QString, QList<HdbData> > &attDataHash,
                                       const QDateTime &startDt,
                                       const QDateTime &stopDt)
{
    QDateTime lastDateTime = stopDt; /* last expected timestamp */
    QDateTime ithDateTime; /* will store the timestamp of the data inside the check cycle below */
    QDateTime expectedDateTime; /* will keep track of the expected date time through the cycle below */
    int expectedDataSize = startDt.secsTo(stopDt) / 60 + 1;

    mMaximum = -100;
    mMinimum = 1000;
    mMinimumPositive = 1000;
    double nanval = nan("NaN");

    int deltaSecs, i, cnt;

    /* find the maximum value
     */
    qDebug() << __FUNCTION__ << "addTadaHash keys" << attDataHash.keys();
    foreach(QString blmattnam, attDataHash.keys())
    {
        const QList<HdbData>& hdbDataList = attDataHash[blmattnam];
        qDebug() << __FUNCTION__ << "hdbDataList size" << hdbDataList.size();
        /* get last timestamp, which is expected at the last location of
         * each dataList
         */
        foreach(HdbData d, hdbDataList)
        {
            if(d.value > mMaximum)
                mMaximum = d.value;
            if(d.value < mMinimum)
                mMinimum = d.value;
            if(d.value > 0 && d.value < mMinimumPositive)
                mMinimumPositive = d.value;
            qDebug() << __FUNCTION__ <<blmattnam <<  d.value << mMaximum << mMinimum;
        }
    }

    qDebug() << __FUNCTION__ << "last timestamp is"
             << lastDateTime << "expected data size" << expectedDataSize;

    /* check all the timestamps inside hdb data list.
     * If data is missing at some time, fill the empty space with an invalid
     * HdbData
     */
    foreach(QString blmattnam, attDataHash.keys())
    {
        /* at the beginning, at the last location we expect the most recent timestamp */
        expectedDateTime = lastDateTime;

        QList<HdbData>& hdbDataList = attDataHash[blmattnam];
        i = hdbDataList.size() - 1;
        cnt = 0;

        if(i < 0)
            qDebug() << __FUNCTION__ << "inflating " << blmattnam << " from size" << hdbDataList.size() << "to size" << expectedDataSize;
        /* a. inflate empty data list with invalid HdbData */
        while(i < 0 && cnt < expectedDataSize) /* empty hdbDataList: no data at all */
        {
            hdbDataList.insert(0, HdbData(expectedDateTime, nanval));
            expectedDateTime = expectedDateTime.addSecs(-DIFFTIME_SECS);
            cnt++;
        }
        if(i < 0) /* must reinitialize data used in while below */
        {
            qDebug() << __FUNCTION__ << " inflated to " <<  hdbDataList.size();
            cnt = 0;
            expectedDateTime = lastDateTime;
        }

        /* b. expand the list of HdbData where necessary.
         */

        /* do not insert a number of elements that would extend the list beyond
         * maxDataLen
         */
        while(i >= 0 && cnt < expectedDataSize)
        {
            ithDateTime = hdbDataList.at(i).timestamp;

            /*
            * calculate the distance, in seconds, between the expected timestamp
            * and the actual timestamp.
            * We will save data only if the two timestamps are separated by at most
            * one minute
            */
            deltaSecs = ithDateTime.secsTo(expectedDateTime);

            if(deltaSecs > DELTA_SECS_TOLERANCE)
            {
                //                qDebug() << __FUNCTION__ << "previous data size for" << blmattnam <<
                //                            attDataHash.value(blmattnam).size();
                /*
                 * insert an invalid HdbData at pos i + 1
                 * hdbDataList is a reference, so we modify it in place,
                 * inside the original hash.
                 */
                hdbDataList.insert(i + 1, HdbData(expectedDateTime, nanval));
//                qDebug() << __FUNCTION__ << "filling " << blmattnam << "pos" << i+1 <<
//                            "expected was" << expectedDateTime << "ith was" << ithDateTime
//                         << "size is now" << attDataHash.value(blmattnam).size();
            }
            else /* look for previous element */
                i--;

            cnt++;
            /*
             * next expected date time (next loop) is one minute before
             */
            expectedDateTime = expectedDateTime.addSecs(-DIFFTIME_SECS);
        }
        /* align the first part of the vector */
        i = expectedDataSize - hdbDataList.size();
        if(i > 0)
            qDebug() << __FUNCTION__ << "--> must push_front" << i << "nan values";
        while(i > 0)
        {
            hdbDataList.push_front(HdbData(expectedDateTime, nanval));
            expectedDateTime = expectedDateTime.addSecs(-DIFFTIME_SECS);
            i--;
        }
    }

}
