#include "exportdata.h"
#include <QFile>
#include <QFileDialog>
#include <QMessageBox>
#include <QTextStream>
#include <QDesktopServices>
#include <QUrl>
#include <QtDebug>

ExportData::ExportData()
{
}

bool ExportData::save(const QHash<QString, QList<HdbData> > &attDataHash,
                      const QStringList &orderedDevices,
                      const QString& attName)
{
    bool ret;
    mError = "";
    QString fileName = QFileDialog::getSaveFileName(0, "Select a file to open", QDir::homePath(), "*.csv");
    if(!fileName.endsWith(".csv"))
        fileName.append(".csv");
    mFilePath = fileName;
    QFile f(fileName);
    if (!f.open(QIODevice::WriteOnly | QIODevice::Text))
    {
         mError = f.errorString();
         ret = false;
    }
    else
    {
        QTextStream out(&f);
        qDebug() << "saving on " << fileName << orderedDevices << "size hash" << attDataHash.size();
        /* write header */
        out << "tango point,blm device,attribute,timestamp,date/time,value\n";
        foreach(QString dev, orderedDevices)
        {
            QString key = dev + "/" + attName;
            QList<HdbData> hdbDataList = attDataHash.value(key);
            qDebug() << key << hdbDataList.size();
            foreach(HdbData hdbd, hdbDataList)
                out << key << "," << dev << "," << attName << ","
                    << hdbd.timestamp.toTime_t() << ","
                    << hdbd.timestamp.toString() << ","
                    << QString::number(hdbd.value) << "\n";
        }
        f.close();
        ret = true;
    }
    return ret;
}

void ExportData::dialogOpenFile()
{
    int result = QMessageBox::question(0, "Would you like to open the file?", "Save successful.\n"
                                       "Would you like to open "
                                       " the file now?",  QMessageBox::Open, QMessageBox::No);
    switch(result)
    {
    case QMessageBox::Open:
        QDesktopServices::openUrl(QUrl("file://" + mFilePath));
        break;
    default:
        break;
    }
}
