#ifndef LABEL_H
#define LABEL_H

#include <QLabel>
#include <QHash>
#include "db/hdbdata.h"
#include <QString>
#include <QList>
#include "painter.h"

class Image;

class Label : public QLabel
{
    Q_OBJECT
public:

    explicit Label(QWidget *parent = 0);
    
    void setData(const QHash<QString, QList<HdbData> > &attDataHash,
                 const QStringList &orderedDeviceList,
                 int maxDataLen,
                 double maximumValue,
                 double minimumValue,
                 double minimumPositive,
                 const QString& blmAttName);

    const QHash<QString, QList<HdbData> > &attDataHash() const;

    const QStringList & orderedBlmDeviceList() const;

    QString blmAttName() const { return mBlmAttName; }

    void draw();

    int maxDataLen() const { return mMaxDataLen; }

    int magnifierAreaWidth() const { return mMagnifierAreaWidth; }

    int magnifierAreaHeight() const { return mMagnifierAreaHeight; }

    double maximumValue() const { return mMaximumValue; }

    double minimumValue() const { return mMinimumValue; }

    double minimumPositive() const { return mMinimumPositive; }

    void invMap(int *blmIndex, int *dataIndex, const QPoint& pos);

    bool drawText() const { return mDrawText; }

    void setPalette(const QString& name, bool update = false);

    int blmGridAlpha() const { return mBlmGridAlpha; }

    bool showBlmGrid() const { return mShowBlmGrid; }

protected:

    void paintEvent(QPaintEvent *);

    void wheelEvent(QWheelEvent *);

    void mousePressEvent(QMouseEvent *ev);

    void mouseMoveEvent(QMouseEvent *ev);

    void keyPressEvent(QKeyEvent *ev);

    void leaveEvent(QEvent *);

    void enterEvent(QEvent *);

signals:

    void dataFixedTime(const QString& timeAsString, const QVector<double> &xData, const QVector<double> &yData);
    void dataFixedBlm(const QString &blmName, const QVector<double> &xData, const QVector<double> & yData);

    void mouseOverLabel(const QString&);
    void mouseOverX1(const QString&);
    void mouseOverX2(const QString&);
    void maxValueUpdated(double max);
    void minValueUpdated(double min);
    void minPositiveUpdated(double minimumPositive);

    void mouseMoved(const QPoint& pos);

    void magnifierRectChanged(const QRect&, const QPixmap *pixmap);

    void magnifierWidthChanged(int w);

    void magnifierHeightChanged(int h);

    void crossColorChanged(const QColor& newColor);

    void timeGridPossible(bool possible);

public slots:

    void setDrawText(bool draw);

    void setShowBlmGrid(bool show);

    void setGridAlpha(int alpha);

    void emptyValueChanged(int valuePercent);

    void setMagnifierEnabled(bool en);

    void setMagnifierAreaWidth(int width);

    void setMagnifierAreaHeight(int height);

    void toggleGridColor();

    void setShowTimeGrid(bool show);

    void setDynamicRange(Painter::DynamicRange dr);

    void setMinExpLog(int exp);

    void setCustomRange(double max, double min);

    void setCustomRangeEnabled(bool en);

private:

    void mMakeJetColorMap();
    void mMakeBWColorMap();
    void mMakeHotColorMap();
    void mMakeDefaultColorMap();

    void mToggleCrossColor();
    void mToggleShowCrossText();

    QRect mMakeMagnifierRect() const;

    int mMaxDataLen;

    int mBlmGridAlpha;

    double mMaximumValue, mMinimumValue, mMinimumPositive;

    QString mBlmAttName;

    QPoint mMousePos;

    QHash<QString, QList<HdbData> > mAttDataHash;
    QStringList mOrderedBlmDeviceList;

    QVector<QRgb> mColorMap;

    Painter *mPainter;

    QString mText, mPaletteName;

    bool mDrawText, mMagnifierEnabled, mShowBlmGrid, mShowTimeGrid, mMouseGrabEnabled;

    int mMagnifierAreaWidth, mMagnifierAreaHeight;

    int mColorIndex, mGridColorIndex;

    bool mLogarithmic, mExponential;

    QList<QColor> mCrossColors;
};

#endif // LABEL_H
