class DotsPainterPrivate;

#include "plot.h"
#include <scenecurve.h>
#include <linepainter.h>
#include <curveitem.h>
#include <dotspainter.h>
#include <elettracolors.h>
#include <timescalelabel.h>
#include <scaleitem.h>
#include <legenditem.h>
#include <markeritem.h>

Plot::Plot(QWidget *parent)
    : PlotSceneWidget(parent)
{
    mType = Undefined;
    this->setLegendVisible(true);
    MarkerItem* marker = new MarkerItem(0);
    installMouseEventListener(marker);
    scene()->addItem(marker);
    marker->setDotRadius(5.0);
}

void Plot::setType(Type t)
{
    mType = t;
    if(t == TimeOnX)
    {
        TimeScaleLabel *timeScaleLabel = new TimeScaleLabel();
        timeScaleLabel->setShowDate(true);
        xScaleItem()->installScaleLabelInterface(timeScaleLabel);
    }
}

Plot::Type Plot::type() const
{
    return mType;
}

void Plot::updateData(const QString& name, const QVector<double> &xData,
                const QVector<double> &yData)
{
    QList<SceneCurve *> curves = getCurves();
    foreach (SceneCurve *c, curves)
        delete c;
    /* add the new curve */
    SceneCurve *c = addCurve(name);
    /* do we want the curves be represented by lines? */
    CurveItem *curveItem = new CurveItem(c);
    scene()->addItem(curveItem);
    c->installCurveChangeListener(curveItem);
    DotsPainter *lp = new DotsPainter(curveItem);
    switch(mType)
    {
    case TimeOnX:
        lp->setDotsColor(KDARKMAROON);
        lp->setBorderColor(KDARKMAROON);
        lp->setRadius(2.0);
        break;
    case BlmOnX:
        lp->setDotsColor(KVERYDARKBLUE);
        lp->setBorderColor(KVERYDARKBLUE);
        lp->setRadius(2.0);
        break;
    default:
        break;
    }
    /* after item painter is installed and color configured */
    this->legendItem()->newCurveAdded(c);
    setData(name, xData, yData);
}
