#include "timeintervalhelper.h"
#include <QWidget>
#include <QPushButton>
#include <QDateTimeEdit>
#include <QRegExp>
#include <QSpinBox>
#include <QCheckBox>

TimeIntervalHelper::TimeIntervalHelper(QWidget *containerWidget, QWidget *mainWidget) :
    QObject(containerWidget)
{
    mWidget = containerWidget;
    mMainWidget = mainWidget;

    QList<QWidget * > widgets;
    widgets = containerWidget->findChildren<QWidget *>(QRegExp("pbLast.*"));
    widgets << containerWidget->findChild<QWidget *>("pbYesterday");
    widgets << containerWidget->findChild<QWidget *>("pbToday");


    widgets << containerWidget->findChild<QSpinBox *>("sbLastHours");

    foreach(QWidget *w, mainWidget->findChildren<QWidget *>(QRegExp("pbFast.*")))
        widgets << w;

    foreach(QWidget *w, widgets)
        connect(w, SIGNAL(clicked()), this, SLOT(widgetClicked()));

    connect(containerWidget->findChild<QSpinBox *>("sbLastHours"), SIGNAL(valueChanged(int)),
            this, SLOT(widgetClicked()));
}

void TimeIntervalHelper::widgetClicked()
{
    QDateTimeEdit *startDte = mWidget->findChild<QDateTimeEdit *>("startDTE");
    QDateTimeEdit *stopDte = mWidget->findChild<QDateTimeEdit *>("stopDTE");
    QCheckBox *showOnClickCb = mWidget->findChild<QCheckBox *>("cbShowDataOnClick");

    QObject *o = sender();
    QString oName = o->objectName();

    const QDateTime now = QDateTime::currentDateTime();

    if(oName.contains("pbLast") && oName.contains("Hrs"))
    {
        oName.remove("pbLast").remove("Hrs");
        int h = oName.toInt();
        stopDte->setDateTime(now);
        QDateTime start = now.addSecs(-60 * 60 * h);
        startDte->setDateTime(start);
        /* immediately refresh after click? */
        if(showOnClickCb->isChecked())
            emit getData();
    }
    else if(oName.contains("Yesterday"))
    {
        QTime t(0, 0, 0);
        QDate d = now.addDays(-1).date();
        startDte->setDateTime(QDateTime(d, t));
        t.setHMS(23, 59, 00);
        stopDte->setDateTime(QDateTime(d, t));
        if(showOnClickCb->isChecked())
            emit getData();
    }
    else if(oName.contains("Today"))
    {
        QTime t(0, 0, 0);
        QDate d = now.date();
        startDte->setDateTime(QDateTime(d, t));
        stopDte->setDateTime(now);
        if(showOnClickCb->isChecked())
            emit getData();
    }
    else if(oName.contains("sbLastHours"))
    {
        QSpinBox *sb = qobject_cast<QSpinBox *>(o);
        QDateTime start = now.addSecs(-60 * 60 * sb->value());
        startDte->setDateTime(start);
        stopDte->setDateTime(now);
    }
    else if(oName.contains("Now"))
    {
        stopDte->setDateTime(now);
    }
    else if(oName.contains("pbFast"))
    {
        QDateTime s1 = startDte->dateTime();
        QDateTime s2 = stopDte->dateTime();

        if(oName == "pbFastBack1Day")
        {
            s1 = s1.addDays(-1);
            s2 = s2.addDays(-1);
        }
        else if(oName == "pbFastBack1Hour")
        {
            s1 = s1.addSecs(-60 * 60);
            s2 = s2.addSecs(-60 * 60);
        }
        else if(oName == "pbFastFwd1Hour")
        {
            s1 = s1.addSecs(60 * 60);
            s2 = s2.addSecs(60 * 60);
        }
        else if(oName == "pbFastFwd1Day")
        {
            s1 = s1.addDays(1);
            s2 = s2.addDays(1);
        }
        startDte->setDateTime(s1);
        stopDte->setDateTime(s2);
        emit getData();
    }

}
