#include "magnifierlabel.h"
#include <math.h>
#include <QImage>
#include <QtDebug>
#include <QPainter>
#include <QtAlgorithms>

MagnifierLabel::MagnifierLabel(QWidget *parent) :
    QLabel(parent)
{
    mCrossColor = Qt::green;
    mCrossAlpha = 120;
}

void MagnifierLabel::setCrossColor(const QColor &c)
{
    mCrossColor = c;
}

void MagnifierLabel::setCrossAlpha(int alpha)
{
    mCrossAlpha = alpha;
}

void MagnifierLabel::magnifyArea(const QRect& r, const QPixmap *pixmap)
{
    if(pixmap)
    {
        /* scale by 0.95 in order not to make the size of the label
         * explode during magnifying.
         */
        QSize size = this->size() * 0.95;
        int rmin = qMin(r.width(), r.height());
        int mymin = qMin(width(), height());
        float side = mymin / (float)rmin;
        QPixmap copy = pixmap->copy(r);
        copy = copy.scaled(size, Qt::KeepAspectRatio);
        /* convert image in order to be able to draw on it.
         */
       // copy = copy.convertToFormat(QImage::Format_RGB32, Qt::AutoColor);
        /* draw the green foresight */
        QPainter p(&copy);
        QColor color = Qt::green;
        color.setAlpha(mCrossAlpha);
        p.setPen(color);
        //p.setRenderHint(QPainter::Antialiasing, true);

        /* horizontal line */
        p.fillRect(0, qRound(copy.height()/2.0),
                   qRound(copy.width()/2.0), floor(side), color);
        p.fillRect(qRound(copy.width()/2.0) + floor(side),
                   qRound(copy.height()/2.0),
                   qRound(copy.width()/2.0) - floor(side/2.0), floor(side), color);
        /* vertical line */
        p.fillRect(qRound(copy.width()/2.0), 0,
                   floor(side), qRound(copy.height()/2.0), color);
        p.fillRect(qRound(copy.width()/2.0), qRound(copy.height()/2.0)
                   + floor(side), floor(side), qRound(copy.height()/2.0) - floor(side), color);
        // QRect r(copy.width()/2 - side/2, copy.height()/2 - side/2, side, side);
        // p.drawRect(r);
        setPixmap(copy);
    }
}
