#include "vlabel.h"
#include <QPainter>
#include <QtDebug>

VLabel::VLabel(QWidget *parent) :
    QLabel(parent)
{

}

void VLabel::setText(const QString & txt)
{
    QLabel::setText(txt);
    updateGeometry();
}

void VLabel::paintEvent(QPaintEvent *e)
{
    const int margin = 4;
    QPainter p(this);
    p.save();

    p.rotate(90);
    int x, y;
    QFontMetrics fm(p.font());

    if(this->alignment() & Qt::AlignBottom)
    {
        x = height() - fm.width(text()) - margin;
    }
    else if(this->alignment() & Qt::AlignTop)
    {
        x = margin;
    }
    else if(this->alignment() & Qt::AlignVCenter)
    {
        x = height()/2 - fm.width(text())/2 - margin / 2;
    }

    if(this->alignment() & Qt::AlignHCenter)
    {
        y = width() / 2 - fm.height()/2;
    }
    else if(this->alignment() & Qt::AlignRight)
    {
        y = width() - margin/2 - fm.height();
    }
    else if(this->alignment() & Qt::AlignLeft)
    {
        y = width() - fm.height() - margin;
    }
    p.translate(x, -y);
    p.drawText(0, 0, text());
    p.restore();
}

QSize VLabel::sizeHint() const
{
    QFontMetrics fm(this->font());
    /* length of the string plus some margin */
    return QSize(fm.height() + 6, fm.width(text()));
}

QSize VLabel::minimumSizeHint() const
{
    QFontMetrics fm(this->font());
    /* length of the string plus some margin */
    return QSize(fm.height() + 4, fm.width(text()));
}

